indexing
	description: "Abstract representation of an option, that might or might not require an extra argument"
	author: "Bernd Schoeller"
	copyright: "(c) 2006 Bernd Schoeller (bernd@fams.de)"
	license: "see included README.txt file"
	date: "$Date: 2006/03/27 08:22:34 $"
	revision: "$Revision: 1.3 $"
	remark: "The preconditions to the creation routines are not verifiable. They should be put into another class."

deferred class OP_OPTION

inherit
	
	OP_WORD_WRAP
	OP_CONSTANTS

feature {NONE} -- Initialization

	initialize is
			-- Perform the common initialization steps.
		do
			description := ""
			reset
		end
		
	make (a_short_form: CHARACTER; a_long_form: STRING) is
			-- Make an option that has `a_short_form' as short form and `a_long_form'
			-- as long form.
		require
			valid_short_form: valid_short_form (a_short_form)
			valid_long_form: valid_long_form (a_long_form)
		do
			set_short_form (a_short_form)
			set_long_form (a_long_form)
			initialize
		ensure
			short_form_set: has_short_form and short_form = a_short_form
			long_form_set: has_long_form and long_form = a_long_form
		end
		
	make_with_short_form (a_short_form: CHARACTER) is
			-- Make an option that has `a_short_form' as short form.
		require
			valid_form: valid_short_form (a_short_form)
		do
			set_short_form (a_short_form)
			initialize
		ensure
			short_form_set: has_short_form and short_form = a_short_form
			no_long_form: not has_long_form
		end
				
	make_with_long_form (a_long_form: STRING) is
			-- Make an option that `a_long_form' as long form.
		require
			valid_form: valid_long_form (a_long_form)
		do
			set_long_form (a_long_form)
			initialize
		ensure
			no_short_form: not has_short_form
			long_form_set: has_long_form and long_form = a_long_form
		end

feature -- Access

	description: STRING
		-- Description of this option

	example: STRING is
			-- Example for the usage of the option (short preferred)
		do
			if has_short_form then
				Result := "["+short_option_introduction.out+short_form.out+"]"
			else
				Result := "["+short_option_introduction.out+long_option_introduction.out+long_form+"]"			
			end
		end

	help_text (indent:INTEGER): STRING is
			-- Help text of this option, calculated from the option_names and the description, assuming
			-- an indention of `indent' is required for correct formating
		require
			indent_not_too_small: indent >= names.count
			indent_not_too_large: indent <= terminal_width
		local
			spaces: STRING
		do
			create spaces.make_filled (' ', indent-names.count)
			Result := word_wrap (names+spaces+description,indent)
		end

	long_form: STRING
		-- Long form
		
	name: STRING is
			-- Name of the option (short or long from)
		do
			if has_long_form then
				Result := short_option_introduction.out+long_option_introduction.out+long_form
			else
				Result := short_option_introduction.out+short_form.out
			end
		end	
	
	names: STRING is
			-- Names of the option (short and long)
		do
			if has_short_form then
				Result := short_option_introduction.out+short_form.out
			end
			if has_long_form then
				if has_short_form then
					Result.append (", "+short_option_introduction.out+long_option_introduction.out+long_form)
				else
					Result := "    "+short_option_introduction.out+long_option_introduction.out+long_form
				end
			end
		end
	
	occurrences: INTEGER is
			-- Number of times this flag was encountered
		deferred
		end
		
	short_form: CHARACTER
		-- Short form
	
feature -- Status report

	has_long_form: BOOLEAN is
			-- Does this option have a long form?
		do
			Result := (long_form /= Void)
		end
		
	has_short_form: BOOLEAN
		-- Does this option have a short form?
	
	needs_parameter: BOOLEAN is
			-- Does this option need a parameter?
		deferred
		end

	was_found: BOOLEAN is
			-- Was the option found during the last parse?
		do
			Result := occurrences > 0
		ensure
			definition: was_found = (occurrences > 0)
		end

feature -- Validity checks
		
	valid_long_form (a_string: STRING): BOOLEAN is
			-- Is `a_string' a valid long form?
			-- Allowed character: alpha_numeric, dash (not twice in a row and not at beginning or end)
		local
			length: INTEGER
			i: INTEGER
		do
			if a_string = Void then
				Result := False
			else
				length := a_string.count
				Result := length >= 1
				from
					i := 1
				until
					i > length or not Result
				loop
					Result := a_string.item (i).is_alpha_numeric or
						(i > 1 and i < length and then 
						 a_string.item (i) = short_option_introduction and a_string.item (i-1) /= short_option_introduction)
					i := i + 1
				end
			end
		ensure
			void_string_are_never_valid: (a_string = Void) implies not Result
		end

	valid_short_form (a_character: CHARACTER): BOOLEAN is
			-- Is `a_character' a valid short form?
			-- Allowed characters: alpha_numeric
		do
			Result := a_character.is_alpha_numeric
		end

feature -- Element change

	set_description (a_string: STRING) is
			-- Let the text `a_string' describe the option.
		require
			not_void: a_string /= Void
		do
			description := a_string
		ensure
			description_set: description = a_string
		end

	set_long_form (a_long_form: STRING) is
			-- Make `a_long_form' the long form.
		require
			valid_form: valid_long_form (a_long_form)
		do
			long_form := a_long_form
		ensure
			long_form_set: long_form = a_long_form
			has_long_form: has_long_form
		end

	set_short_form (a_short_form: CHARACTER) is
			-- Make `a_short_form' the short form.
		require
			valid_form: valid_short_form (a_short_form)
		do
			short_form := a_short_form
			has_short_form := True
		ensure
			short_form_set: short_form = a_short_form
			has_short_form: has_short_form
		end
		
feature -- Removal

	remove_long_form is
			-- Remove the long form.
		require
			has_short_form: has_short_form
		do
			long_form := Void
		ensure
			long_form_remove: not has_long_form
		end

	remove_short_form is
			-- Remove the short form.
		require
			has_long: has_long_form
		do
			has_short_form := False
		ensure
			short_form_remove: not has_short_form
		end
		
feature {OP_PARSER} -- Parser Interface

	record_occurrence (a_parser: OP_PARSER) is
			-- This option was found during parsing by `a_parser'.
		require
			parser_available: a_parser /= Void
		deferred
		ensure
			occurrences_increased: occurrences = old occurrences + 1
		end
		
	reset is
			-- Reset the option to a clean state before parsing.
		deferred
		ensure
			not_found: not was_found
		end
		
invariant

	has_short_form_requires_valid_form: has_short_form implies valid_short_form (short_form)
	has_long_form_requires_valid_form: has_long_form implies valid_long_form (long_form)
	has_short_or_long: has_long_form or has_short_form
	occurrences_positive: occurrences >= 0

end
