indexing
	description: "OP argument parser"
	author: "Bernd Schoeller"
	copyright: "(c) 2006 Bernd Schoeller (bernd@fams.de)"
	license: "see included README.txt file"
	date: "$Date: 2006/03/27 08:22:34 $"
	revision: "$Revision: 1.3 $"

class OP_PARSER

inherit
	
	KL_SHARED_ARGUMENTS
	KL_SHARED_STANDARD_FILES
	KL_SHARED_EXCEPTIONS
	OP_WORD_WRAP
	OP_CONSTANTS

create

	make

feature {NONE} -- Initialization

	make is
			-- Create an argument parser.
		do
			create {DS_LINKED_LIST[OP_OPTION]}options.make
			create help_option.make (help_text_short_option,help_text_long_option)
			help_option.set_description (help_text_description)
			extend (help_option)
			parameters_description := default_parameters_description
			create error_handler.make_standard
		end

feature -- Access

	error_handler: OP_ERROR_HANDLER

	help_option: OP_DISPLAY_HELP_FLAG
		-- Automatically generated option to display the help text

	options: DS_LIST [OP_OPTION]
		-- List of all available options

	parameters: DS_LIST [STRING]
		-- All argument that were not part of any option
			
	parameters_description: STRING
		-- Description of the parameters
		
	required_parameters: INTEGER
		-- Number of required parameters
		
	last_option_parameter: STRING
		-- Last parameter that was passed to an option
			
feature -- Status report

	has_long_option (an_option: STRING): BOOLEAN is
			-- Does the parser have a long option `o'?
		require
			not_void: an_option /= Void
		do
			from
				options.start
			until
				options.off or Result
			loop
				if options.item_for_iteration.has_long_form then
					Result := options.item_for_iteration.long_form.is_equal (an_option)
				end
				options.forth
			end		
		end

	has_short_option (a_character: CHARACTER): BOOLEAN is
			-- Does the parser have a short option `a_character'?
		do
			from
				options.start
			until
				options.off or Result
			loop
				if options.item_for_iteration.has_short_form then
					Result := options.item_for_iteration.short_form = a_character
				end
				options.forth
			end		
		end
	
feature -- Error handling

	final_error_action is
			-- Code to execute when an error was encountered.
		do
			error_handler.report_error_message (abort_message)
			exceptions.die (1)
		end

feature -- Configuration

	set_error_handler (an_error_handler: OP_ERROR_HANDLER) is
			-- Set the error handler to `an_error_handler'.
		require
			not_void: an_error_handler /= Void
		do
			error_handler := an_error_handler
		ensure
			error_handler_set: error_handler = an_error_handler
		end

	set_parameters_description (a_string: STRING) is
			-- Set the description for parameters to `a_string'.
		require
			not_void: a_string /= Void
		do
			parameters_description := a_string
		ensure
			name_set: parameters_description = a_string
		end
		
	set_required_parameters (a_value: INTEGER) is
			-- The parser wants at least `a_value' parameters.
		require
			not_too_small: a_value >= 0
		do
			required_parameters := a_value
		ensure
			value_set: required_parameters = a_value
		end

feature -- Option Management

	extend (an_option: OP_OPTION) is
			-- Add `an_option' to the list of options.
		require
			not_void: an_option /= Void
			not_short_option_taken: an_option.has_short_form implies not has_short_option (an_option.short_form)
			not_long_option_taken: an_option.has_long_form implies not has_long_option (an_option.long_form)
			not_already_available: not options.has (an_option)
		do
			options.force_last (an_option)
		ensure
			added: options.has (an_option)
		end
		
	remove (an_option: OP_OPTION) is
			-- Remove `an_option' from the list of options.
		require
			has_option: options.has (an_option)
		do
			options.delete (an_option)
		ensure
			removed: not options.has (an_option)
		end
		
feature -- Parser

	parse_arguments is
			-- Parse the command line arguments.
		local
			args: DS_ARRAYED_LIST [STRING]
		do
			create args.make_from_array (arguments.argument_array)
			args.remove_first
			parse_list (args)
		end
		
	parse_list (a_list: DS_LIST [STRING]) is
			-- Parse `a_list'.
		require
			not_void: a_list /= Void
		local
			error: OP_ERROR
		do
			reset_parser
			create {DS_LINKED_LIST [STRING]}parameters.make
			argument_list := a_list
			from
				argument_list.start
			until
				argument_list.off
			loop
				parse_argument
			end
			if parameters.count < required_parameters then
				create error.make_not_enough_parameters (parameters.count,required_parameters)
				error_handler.report_error (error)
			end			
			if error_handler.has_error then
				final_error_action
			end
		end
		
feature {NONE} -- Implementation

	argument_list: DS_LIST [STRING]
	
	parse_argument is
			-- Parse the current argument.
		local
			argument: STRING
			length: INTEGER
		do
			argument := argument_list.item_for_iteration
			length := argument.count
			if length >= 2 and then argument.item (1) = short_option_introduction then
				if argument.item (2) = long_option_introduction then
					if length = 2 then
						from
							argument_list.forth
						until
							argument_list.off
						loop
							parameters.force_last (argument_list.item_for_iteration)
							argument_list.forth
						end
					else
						parse_long
					end
				else
					parse_short
				end
			else
				parameters.force_last (argument)
				argument_list.forth
			end
		end

	parse_long is
			-- Parse a long argument
		require
			current_is_long_option: argument_list.item_for_iteration.count > 2 and then
			                        (argument_list.item_for_iteration.item (1) = short_option_introduction and
			                         argument_list.item_for_iteration.item (2) = long_option_introduction)
		local
			argument: STRING
			option_string: STRING
			option: OP_OPTION
			parameter_position: INTEGER
			parameter: STRING
			error: OP_ERROR
		do
			argument := argument_list.item_for_iteration
			parameter_position := argument.index_of (long_option_parameter_introduction, 3)
			if parameter_position > 0 then
				option_string := argument.substring (3, parameter_position-1)
				parameter := argument.substring (parameter_position+1, argument.count)
			else
				option_string := argument.substring (3, argument.count)
			end
			from
				option := Void
				options.start
			until
				option /= Void or options.off
			loop
				if options.item_for_iteration.has_long_form and then
				   options.item_for_iteration.long_form.is_equal (option_string) then
				   option := options.item_for_iteration
				end
				options.forth
			end
			if option = Void then
				create error.make_unknown_option_error (option_string)
				error_handler.report_error (error)
			else
				if option.needs_parameter then
					if parameter = Void then
						argument_list.forth
						if argument_list.off then
							create error.make_missing_parameter_error (option)
							error_handler.report_error (error)
						else
							parameter := argument_list.item_for_iteration
						end
					end
					if parameter /= Void then
						last_option_parameter := parameter
						option.record_occurrence (Current)				
					end
				else
					if parameter = Void then	
						option.record_occurrence (Current)
					else
						create error.make_unnecessary_parameter_error (option, parameter)
						error_handler.report_error (error)
					end
				end
			end
			if not argument_list.off then
				argument_list.forth				
			end
		end
		
	parse_short is
			-- Parse a short option.
		local
			argument: STRING
			i: INTEGER
			option_character: CHARACTER
			option: OP_OPTION
			parameter: STRING
			error: OP_ERROR
		do
			argument := argument_list.item_for_iteration
			from			
				i := 2
			until
				i > argument.count
			loop
				option_character := argument.item(i)
				from
					option := Void
					options.start
				until
					option /= Void or options.off
				loop
					if options.item_for_iteration.has_short_form and then
					   options.item_for_iteration.short_form = option_character then
					   option := options.item_for_iteration
					end
					options.forth
				end
				if option = Void then
					create error.make_unknown_option_error (option_character.out)
					error_handler.report_error (error)
				else
					if option.needs_parameter then
						if i = argument.count then
							argument_list.forth
							if not argument_list.off then
								parameter := argument_list.item_for_iteration
							end	
						else
							parameter := argument.substring (i+1,argument.count)
							i := argument.count
						end
						if parameter = Void then
							create error.make_missing_parameter_error (option)
							error_handler.report_error (error)
						else
							last_option_parameter := parameter
							option.record_occurrence (Current)
						end
					else
						option.record_occurrence (Current)
					end
				end
				i := i + 1
			end
			if not argument_list.off then
				argument_list.forth
			end
		end
		
	reset_parser is
			-- Reset parser and all option prior to parsing arguments.
		do
			error_handler.reset
			from
				options.start
			until
				options.off
			loop
				options.item_for_iteration.reset
				options.forth
			end
		ensure
			not_has_errors: not error_handler.has_error
		end

invariant

	options_not_void: options /= Void

end
